#ifndef REACTOREVENT_HH
#define REACTOREVENT_HH
#include <math.h>
#include <iostream.h>
#include "ReactorFortran.hh"
#include "ReactorXC.hh"


class ReactorEvent{

public:

  ReactorEvent();
  ~ReactorEvent();
  ReactorEvent(const ReactorEvent& Event);
  ReactorEvent& operator=(const ReactorEvent& rhs);
  //
  // for reweighting-- not implemented
  //
  double XCweight();
  double XCweight(double Enu);
  double XCweight(ReactorEvent& Event);

  double GetXCWeight() const {return XCWeight;}
  double GetFluxWeight() const {return FluxWeight;}
  double GetEnu() const {return Pnubar[0];}
  double GetPositronKE() const {return Ppositron[4];}
  double GetCosThetaLab() const {return Ppositron[3]/Ppositron[5];}
  double GetNeutronKE()const {return Pneutron[4];}
  double GetNeutronCosTheta() const {return Pneutron[3]/Pneutron[5];}

  double GetInteractionTime() const {return Rnubar[0];}
  double GetVertexR() const {return Rnubar[4];}
  double GetVertexCosTheta() const {return Rnubar[5];}
  double GetVertexPhi() const {return Rnubar[6];}
  double* const GetVertexXYZ() {return Rnubar+1;}

private:

  double XCWeight;
  double FluxWeight;

  static const int Pdim = 6;
  double Pnubar[Pdim];
  double Pneutron[Pdim];
  double Ppositron[Pdim];

  static const int Rdim = 7;
  double Rnubar[Rdim];
  double Rneutron[Rdim];
  double Rpositron[Rdim];

  int Ngamma;
  double* Pgamma;

  static const int XCorder = 1;

  static const double Emin=1.85;
  static const double Emax=8.15;
  static const double Rmaxgen = 1.0;

  static const double pi = 3.1415928;
  static const double Mproton = 938.27200;
  static const double Delta = 1.2933318;
  static const double Melectron = 0.510998902;
};

ReactorEvent::ReactorEvent(){

  XCWeight = 1;
  Ngamma = 0;
  Pgamma = new double[Ngamma*Pdim];
  
  double XCmax = InverseBeta(Emax,-1);
  //  double FluxMax = U235flux(Emin);
  double FluxMax = RMPflux(Emin);

  double Enu,z,phi;
  double R,zR,phiR;
  bool passed=0;


  while(!passed){
    float r[7];
    int len = 7;
    ranlux_(r,&len);
    Enu = Emin+(Emax-Emin)*r[0];
    z = -1+2*r[1];
    phi = 2*pi*r[2];

    R = Rmaxgen*exp(log(r[3])/3);
    zR = -1+2*r[4];
    phiR = 2*pi*r[5];

    float XCtest = XCmax*FluxMax*r[6];
    XCWeight = InverseBeta(Enu,z,XCorder);
    FluxWeight=RMPflux(Enu);
    passed= XCWeight*FluxWeight>XCtest;
  }
  

  double E0 = Enu - Delta;
  double p0 = sqrt(E0*E0-Melectron*Melectron);
  double v0 = p0/E0;
  double Ysquared = (Delta*Delta-Melectron*Melectron)/2;
  double E1 = E0*(1-Enu/Mproton*(1-v0*z))-Ysquared/Mproton;
  double p1 = sqrt(E1*E1-Melectron*Melectron);

  Pnubar[0] = Enu;
  Pnubar[1] = 0;
  Pnubar[2] = 0;
  Pnubar[3] = Enu;
  Pnubar[4] = Enu;
  Pnubar[5] = Enu;

  Ppositron[0] = E1;
  Ppositron[1] =  p1*sqrt(1-z*z)*cos(phi);
  Ppositron[2] =  p1*sqrt(1-z*z)*sin(phi);
  Ppositron[3] =  p1*z;
  Ppositron[4] =  E1-Melectron;
  Ppositron[5] =  p1;

  Pneutron[0] = Enu+Mproton-E1;
  Pneutron[1] = -Ppositron[1];
  Pneutron[2] = -Ppositron[2];
  Pneutron[3] = Pnubar[3]-Ppositron[3];
  Pneutron[4] = Pneutron[0]-Mproton-Delta;
  Pneutron[5] = sqrt(Pneutron[0]*Pneutron[0]-
		     (Mproton+Delta)*(Mproton+Delta));

  Rnubar[0] = 0;
  Rnubar[1] = R*sqrt(1-zR*zR)*cos(phiR);
  Rnubar[2] = R*sqrt(1-zR*zR)*sin(phiR);
  Rnubar[3] = R*zR;
  Rnubar[4] = R;
  Rnubar[5] = zR;
  Rnubar[6] = phiR;

  for(int n=0;n<Rdim;n++){
    Rneutron[n] = Rnubar[n];
    Rpositron[n] = Rnubar[n];
  }
}

ReactorEvent::~ReactorEvent(){
  delete[] Pgamma;
}

ReactorEvent::ReactorEvent(const ReactorEvent& Event){

  XCWeight = Event.XCWeight;
  FluxWeight = Event.FluxWeight;

  for(int n=0;n<Pdim;n++){
    Pnubar[6]=Event.Pnubar[n];
    Pneutron[6]=Event.Pneutron[n];
    Ppositron[6]=Event.Ppositron[n];
  }
  for(int n=0;n<Rdim;n++){
    Rnubar[n]=Event.Rnubar[n];
    Rneutron[n]=Event.Rneutron[n];
    Rpositron[n]=Event.Rpositron[n];
  }

  Ngamma = Event.Ngamma;
  Pgamma = new double[Ngamma];
  double* pold = Event.Pgamma+Pdim*Ngamma;
  double* pnew = Pgamma+Pdim*Ngamma;
  while(pnew>pold)--pnew=--pold;

}    

ReactorEvent& ReactorEvent::operator=(const ReactorEvent& rhs){

  if(this == &rhs)return *this;

  XCWeight = rhs.XCWeight;
  FluxWeight = rhs.FluxWeight;
  
  for(int n=0;n<Pdim;n++){
    Pnubar[6]=rhs.Pnubar[n];
    Pneutron[6]=rhs.Pneutron[n];
    Ppositron[6]=rhs.Ppositron[n];
  }
  for(int n=0;n<Rdim;n++){
    Rnubar[n]=rhs.Rnubar[n];
    Rneutron[n]=rhs.Rneutron[n];
    Rpositron[n]=rhs.Rpositron[n];
  }
  
  delete[] Pgamma;
  Ngamma = rhs.Ngamma;
  Pgamma = new double[Ngamma];
  double* pold = rhs.Pgamma+Pdim*Ngamma;
  double* pnew = Pgamma+Pdim*Ngamma;
  while(pnew>pold)--pnew=--pold;

  return *this;

}    

#endif






